@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Modules::AgentTicketService;
@@ -200,6 +200,9 @@ sub Run {
         $Output .= $LayoutObject->NavigationBar();
     }
 
+    # Notify if there are tickets which are not updated.
+    $Output .= $LayoutObject->NotifyNonUpdatedTickets();
+
     # viewable locks
     my @ViewableLockIDs = $Kernel::OM->Get('Kernel::System::Lock')->LockViewableLock( Type => 'ID' );
 
@@ -317,13 +320,13 @@ sub Run {
 
     # get personal page shown count
     my $PageShownPreferencesKey = 'UserTicketOverview' . $View . 'PageShown';
-    my $PageShown = $Self->{$PageShownPreferencesKey} || 10;
+    my $PageShown               = $Self->{$PageShownPreferencesKey} || 10;
 
     # do shown tickets lookup
     my $Limit = 10_000;
 
     my $ElementChanged = $ParamObject->GetParam( Param => 'ElementChanged' ) || '';
-    my $HeaderColumn = $ElementChanged;
+    my $HeaderColumn   = $ElementChanged;
     $HeaderColumn =~ s{\A ColumnFilter }{}msxg;
 
     # get data (viewable tickets...)
@@ -502,7 +505,7 @@ sub Run {
     $Data{TicketsShown} = $Count || 0;
 
     # Get ticket count for all services.
-    my @ServiceIDs = sort { $AllServices{$a} cmp $AllServices{$b} } keys %AllServices;
+    my @ServiceIDs           = sort { $AllServices{$a} cmp $AllServices{$b} } keys %AllServices;
     my @AllServicesTicketIDs = $TicketObject->TicketSearch(
         LockIDs    => \@ViewableLockIDs,
         StateIDs   => \@ViewableStateIDs,
@@ -676,7 +679,7 @@ sub _MaskServiceView {
             $Counter{$CustomService} = $Counter{ $Service{Service} };
             $Service{Service} = $CustomService;
         }
-        my @ServiceName = split /::/, $Service{Service};
+        my @ServiceName      = split /::/, $Service{Service};
         my $ShortServiceName = $ServiceName[-1];
         $Service{ServiceID} = 0 if ( !$Service{ServiceID} );
 
